@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2021 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2016-2022 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
@@ -21,8 +21,9 @@
 
 static void test(void)
 {
-	knet_handle_t knet_h;
+	knet_handle_t knet_h1, knet_h[2];
 	int logfds[2];
+	int res;
 	unsigned int interval;
 
 	printf("Test knet_handle_pmtud_getfreq incorrect knet_h\n");
@@ -34,40 +35,21 @@ static void test(void)
 
 	setup_logpipes(logfds);
 
-	knet_h = knet_handle_start(logfds, KNET_LOG_DEBUG);
+	knet_h1 = knet_handle_start(logfds, KNET_LOG_DEBUG, knet_h);
 
 	flush_logs(logfds[0], stdout);
 
 	printf("Test knet_handle_pmtud_getfreq with no interval\n");
-	if ((!knet_handle_pmtud_getfreq(knet_h, NULL)) || (errno != EINVAL)) {
-		printf("knet_handle_pmtud_getfreq accepted invalid interval or returned incorrect error: %s\n", strerror(errno));
-		knet_handle_free(knet_h);
-		flush_logs(logfds[0], stdout);
-		close_logpipes(logfds);
-		exit(FAIL);
-	}
+	FAIL_ON_SUCCESS(knet_handle_pmtud_getfreq(knet_h1, NULL), EINVAL);
 
-	flush_logs(logfds[0], stdout);
+	FAIL_ON_ERR(knet_handle_pmtud_getfreq(knet_h1, &interval));
 
-	if (knet_handle_pmtud_getfreq(knet_h, &interval) < 0) {
-		printf("knet_handle_pmtud_getfreq failed error: %s\n", strerror(errno));
-		knet_handle_free(knet_h);
-		flush_logs(logfds[0], stdout);
-		close_logpipes(logfds);
-		exit(FAIL);
-	}
-
-	if (knet_h->pmtud_interval != interval) {
+	if (knet_h1->pmtud_interval != interval) {
 		printf("knet_handle_pmtud_getfreq failed to set the value\n");
-		knet_handle_free(knet_h);
-		flush_logs(logfds[0], stdout);
-		close_logpipes(logfds);
-		exit(FAIL);
+		CLEAN_EXIT(FAIL);
 	}
 
-	knet_handle_free(knet_h);
-	flush_logs(logfds[0], stdout);
-	close_logpipes(logfds);
+	CLEAN_EXIT(CONTINUE);
 }
 
 int main(int argc, char *argv[])
